from cnmpdk.ip_blocks.cnmD2SBB_1to1 import (
    cnmD2SBB_1to1,
)

from cnmpdk.ip_blocks.cnmD2SBB_1to2 import (
    cnmD2SBB_1to2,
)

from cnmpdk.ip_blocks.cnmD2SBB_1to3 import (
    cnmD2SBB_1to3,
)

from cnmpdk.ip_blocks.cnmINVTAP_10_0 import (
    cnmINVTAP_10_0,
)

from cnmpdk.ip_blocks.cnmMMI1x2DE_BB import (
    cnmMMI1x2DE_BB,
)

from cnmpdk.ip_blocks.cnmMMI1x2SH_BB import (
    cnmMMI1x2SH_BB,
)

from cnmpdk.ip_blocks.cnmMMI2x2DE_BB import (
    cnmMMI2x2DE_BB,
)

from cnmpdk.ip_blocks.cnmMMI2x2SH_BB import (
    cnmMMI2x2SH_BB,
)

from cnmpdk.ip_blocks.cnmMMI3x3DE_BB import (
    cnmMMI3x3DE_BB,
)

from cnmpdk.ip_blocks.cnmMMI3x3SH_BB import (
    cnmMMI3x3SH_BB,
)

from cnmpdk.ip_blocks.cnmMMI1585_BB import (
    cnmMMI1585_BB,
)


from cnmpdk.ip_blocks.cnmMMI8515_BB import (
    cnmMMI8515_BB,
)

from cnmpdk.ip_blocks.cnmS2DBB_1to1 import (
    cnmS2DBB_1to1,
)

from cnmpdk.ip_blocks.cnmS2DBB_2to1 import (
    cnmS2DBB_2to1,
)

from cnmpdk.ip_blocks.cnmS2DBB_3to1 import (
    cnmS2DBB_3to1,
)

__all__ = [
    "cnmD2SBB_1to1",
    "cnmD2SBB_1to2",
    "cnmD2SBB_1to3",
    "cnmINVTAP_10_0",
    "cnmMMI1x2DE_BB",
    "cnmMMI1x2SH_BB",
    "cnmMMI2x2DE_BB",
    "cnmMMI2x2SH_BB",
    "cnmMMI3x3DE_BB",
    "cnmMMI3x3SH_BB",
    "cnmMMI1585_BB",
    "cnmMMI8515_BB",
    "cnmS2DBB_1to1",
    "cnmS2DBB_2to1",
    "cnmS2DBB_3to1",
]


from __future__ import annotations
import gdsfactory as gf
from cnmpdk.config import PATH
from cnmpdk.tech import LAYER


@gf.cell
def cnmINVTAP_10_0() -> gf.Component:
    """Return cnmINVTAP_10 fixed cell."""
    c = gf.Component()
    c = gf.import_gds(PATH.library_path, "cnmINVTAP_10.0")
    c.add_port(name="in0",center=[0,0],width=1.2,orientation=180,layer=LAYER.PORT)
    c.add_port(name="out0",center=[425.12500,0],width=0.95,orientation=0,layer=LAYER.PORT)
    return c


from __future__ import annotations
import gdsfactory as gf
from cnmpdk.config import PATH

@gf.cell
def cnmS2DBB_3to1() -> gf.Component:
    """Return cnmS2DBB_3to1 fixed cell."""
    c = gf.Component()
    c = gf.import_gds(PATH.library_path, "cnmS2DBB_3to1")
    c.add_port(name="in0",center=[0,0],width=3.2,orientation=180,layer=1)
    c.add_port(name="out0",center=[83,0],width=1.2,orientation=0,layer=1)
    return c


from __future__ import annotations
import gdsfactory as gf
from cnmpdk.config import PATH
from cnmpdk.tech import LAYER
@gf.cell
def cnmMMI1x2DE_BB() -> gf.Component:
    """Return cnmMMI1x2DE_BB fixed cell."""
    c = gf.Component()
    c = gf.import_gds(PATH.library_path, "cnmMMI1x2DE_BB")
    c.add_port(name="in0",center=[0,0],width=1.2,orientation=180,layer=LAYER.PORT)
    c.add_port(name="out0",center=[66.08700,-3.85],width=1.2,orientation=0,layer=LAYER.PORT)
    c.add_port(name="out1",center=[66.08700,3.85],width=1.2,orientation=0,layer=LAYER.PORT)
    return c


from __future__ import annotations
import gdsfactory as gf
from cnmpdk.config import PATH

@gf.cell
def cnmMMI3x3DE_BB() -> gf.Component:
    """Return cnmMMI3x3DE_BB fixed cell."""
    c = gf.Component()
    c = gf.import_gds(PATH.library_path, "cnmMMI3x3DE_BB")
    c.add_port(name="in0",center=[0,-5.65],width=1.2,orientation=180,layer=1)
    c.add_port(name="in1",center=[0,0],width=1.2,orientation=180,layer=1)
    c.add_port(name="in2",center=[0,5.65],width=1.2,orientation=180,layer=1)
    c.add_port(name="out0",center=[210.47700,-5.65],width=1.2,orientation=0,layer=1)
    c.add_port(name="out1",center=[210.47700,0],width=1.2,orientation=0,layer=1)
    c.add_port(name="out1",center=[210.47700,5.65],width=1.2,orientation=0,layer=1)
    return c


from cnmpdk.components.cnm_die_10x5 import (
    cnm_die_10x5,
)

from cnmpdk.components.cnm_straight import (
    cnm_straight,
)

from cnmpdk.components.cnm_bend_circular import (
    cnm_bend_circular,
)

from cnmpdk.components.cnm_sagnac import (
    cnm_sagnac,
)
__all__ = [
    "cnm_die_10x5",
    "cnm_straight",
    "cnm_bend_circular",
    "cnm_sagnac",
]


"""cnmpdk straight waveguide."""
from __future__ import annotations

import gdsfactory as gf
from gdsfactory.component import Component
from gdsfactory.cross_section import CrossSectionSpec
import warnings

@gf.cell
def cnm_straight(
    length: float = 10.0,
    npoints: int = 2,
    width: float | None = None,
    cross_section: CrossSectionSpec = "cnm_deep",
    layer: gf.typings.LayerSpec | None = None,
) -> Component:
    """Returns a Straight waveguide.

    Args:
        length: straight length (um).
        npoints: number of points.
        layer: layer to use. Defaults to cross_section.layer.
        width: width to use. Defaults to cross_section.width.
        add_pins: add pins to the component.
        cross_section: specification (CrossSection, string or dict).

    .. code::

        o1 -------------- o2
                length
    """
    x = gf.get_cross_section(cross_section)

    if width or x.width < 0.6:
       raise ValueError(f"The litho resolution is 0.6")
    
    c = Component()
    c.add_ref(gf.components.straight(length=length, npoints=npoints, cross_section=cross_section))

    return c

if __name__ == "__main__":
    import gdsfactory as gf

    c = cnm_straight(cross_section="cnm_deep")
    # c = straight()
    print(c.info)
    c.show(show_ports=True)


import gdsfactory as gf

from cnmpdk.config import PATH

import gdsfactory as gf
from pydantic import BaseModel
from gdsfactory.typings import Layer
from functools import partial
from gdsfactory.technology import LayerLevel, LayerStack
from gdsfactory.cross_section import cross_section

# Define PDK Materials
class LayerMap(BaseModel):
    SHALLOW_WAVEGUIDES: Layer = (1, 0)
    DEEP_WAVEGUIDES: Layer = (2, 0)
    TRENCH: Layer = (3, 0)
    HEATER: Layer = (4, 0)
    DEEP_WAVEGUIDES_PROTECTION: Layer = (11, 0)
    SHALLOW_WAVEGUIDES_PROTECTION: Layer = (21, 0)
    DICING_LINES: Layer = (100, 0)
    PORT: Layer = (1,10)
    BB_OUTLINE: Layer = (51,0)
    BB_GUIDES: Layer = (52,0)
    BB_PARAMETERS: Layer = (53,0)
    
LAYER = LayerMap()

# Define XSections
deep = gf.partial(
    gf.cross_section.strip,
    width=1.2,
    radius=100,
    layer=LAYER.DEEP_WAVEGUIDES,
    # cladding_layers=(LAYER.DEEP_WAVEGUIDES,LAYER.DEEP_WAVEGUIDES_PROTECTION),      TODO: Delete or fix
    # cladding_offsets=(0.1,15,-15,-0.1),                                            TODO: Delete or fix
)
shallow = gf.partial(
    gf.cross_section.strip,
    width=1.2,
    radius=150,
    layer=LAYER.SHALLOW_WAVEGUIDES,
    # cladding_layers=(LAYER.SHALLOW_WAVEGUIDES,LAYER.SHALLOW_WAVEGUIDES_PROTECTION), TODO: Delete or fix
    # cladding_offsets=(0.1,15,-15,-0.1),                                             TODO: Delete or fix
)
heater = gf.partial(
    cross_section,
    width=10,
    layer=LAYER.HEATER,
    # name="cnm_heater",TODO: Delete or fix
    # add_bbox=None,    TODO: Delete or fix
)
trench = gf.partial(
    cross_section,
    width=10,
    layer=LAYER.TRENCH,
    # name="cnm_trench", TODO: Delete or fix
    # add_bbox=None,     TODO: Delete or fix
)

cnm_cross_sections = dict( cnm_deep=deep, cnm_trench=trench, cnm_heater=heater, cnm_shallow=shallow)

nm = 1e-3
um = 1

def get_layer_stack_cnmpdk(
    Thermal_SiO2_thickness: float = -2.5*um,
    zmin_Thermal_SiO2: float = -2.5*um,

    LPCVD_Si3N4_thickness: float = 300 * nm,
    zmin_LPCVD_Si3N4: float = 0.0,

    PECVD_SiO2_thickness: float = -2*um,
    zmin_PECVD_SiO2: float = 300*nm,

    thickness_Cr: float = 10 * nm,
    zmin_Cr: float = 300*nm + 2*um,

    thickness_Au: float = 90*nm,
    zmin_Au: float = 300*nm + 2*um + 90*nm,
) -> LayerStack:
    """Returns cnmpdk LayerStack"""

    return LayerStack(
        layers=dict(
            # substrate=LayerLevel(
            #     layer=LAYER.DICING_LINES,
            #     thickness=substrate_thickness,
            #     zmin=-substrate_thickness - box_thickness,
            #     material="Si",
            #     mesh_order=101,
            #     background_doping={"concentration": "1E14", "ion": "Boron"},
            #     orientation="100",
            # ),
            box=LayerLevel(
                layer=LAYER.DICING_LINES,
                thickness=Thermal_SiO2_thickness,
                zmin=zmin_Thermal_SiO2,
                material="Thermal_SiO2",
                mesh_order=9,
            ),

            wg=LayerLevel(
                layer=LAYER.DEEP_WAVEGUIDES,
                thickness=LPCVD_Si3N4_thickness,
                zmin=0,
                material="LPCVD_Si3N4",
            ),

            metal1=LayerLevel(
                layer=LAYER.HEATER,
                thickness=thickness_Cr,
                zmin=zmin_Cr,
                material="Cr",
                mesh_order=2,
            ),
            metal2=LayerLevel(
                layer=LAYER.HEATER,
                thickness=thickness_Au,
                zmin=zmin_Au,
                material="Au",
                mesh_order=2,
            ),
        )
    )
               

LAYER_STACK = get_layer_stack_cnmpdk()


"""cnmpdk straight waveguide."""
from __future__ import annotations

import gdsfactory as gf
from gdsfactory.component import Component
from gdsfactory.cross_section import CrossSectionSpec
import cnmpdk
import warnings
from gdsfactory.components.bend_euler import bend_euler
from gdsfactory.typings import (
    ComponentSpec,
    CrossSectionSpec,
)
import math

@gf.cell
def cnm_sagnac(
    radius: float = 100.0,
    bend_spec: ComponentSpec = bend_euler,
    cross_section: CrossSectionSpec = "cnm_deep",
    layer: gf.typings.LayerSpec | None = None,
) -> Component:
    """Returns a Straight waveguide.

    Args:
        length: straight length (um).
        npoints: number of points.
        layer: layer to use. Defaults to cross_section.layer.
        width: width to use. Defaults to cross_section.width.
        add_pins: add pins to the component.
        cross_section: specification (CrossSection, string or dict).

    .. code::

        o1 -------------- o2
                length
    """
    c = gf.Component()
    mmi = c << cnmpdk.ip_blocks.cnmMMI2x2DE_BB()

    # phi_rad = math.acos(1+(9-2*radius)/(4*radius))
    # phi = phi_rad*180/math.pi
    # bend_circ = gf.components.bend_circular(angle=phi, cross_section=cross_section, add_pins=True)
    # bend_bot = c.add_ref(bend_circ).connect("o1", destination=mmi.ports["in0"])
    # bend_top = c.add_ref(bend_circ).connect("o1", destination=mmi.ports["in1"]).mirror_y(y0=4.5)
    # route = gf.routing.get_route(mmi.ports["in0"], mmi.ports["in1"], radius=radius, cross_section=cross_section)
    # TODO: bend_spec

    route = gf.routing.get_route(mmi.ports["in0"], mmi.ports["in1"], radius=radius, cross_section=cross_section)
    c.add(route.references)
    c.add_port("o1", port=mmi.ports["out0"])
    c.add_port("o2", port=mmi.ports["out1"])

    return c

if __name__ == "__main__":
    import gdsfactory as gf

    c = cnm_sagnac(cross_section="cnm_deep")
    # c = straight()
    print(c.info)
    c.show(show_ports=True)



from __future__ import annotations

from functools import partial

import gdsfactory as gf
from gdsfactory.component import Component
from gdsfactory.path import arc
from gdsfactory.snap import snap_to_grid
from gdsfactory.typings import CrossSectionSpec


@gf.cell
def cnm_bend_circular(
    radius: float | None = None,
    angle: float = 90.0,
    npoints: int | None = None,
    layer: gf.typings.LayerSpec | None = None,
    width: float | None = None,
    cross_section: CrossSectionSpec = "xs_sc",
    add_pins: bool = True,
) -> Component:
    """Returns a radial arc.

    Args:
        radius: in um. Defaults to cross_section_radius.
        angle: angle of arc (degrees).
        npoints: number of points.
        layer: layer to use. Defaults to cross_section.layer.
        width: width to use. Defaults to cross_section.width.
        cross_section: spec (CrossSection, string or dict).
        add_pins: add pins to the component.

    .. code::

                  o2
                  |
                 /
                /
               /
       o1_____/
    """
    x = gf.get_cross_section(cross_section)
    radius = radius or x.radius
    if layer or width:
        x = x.copy(layer=layer or x.layer, width=width or x.width)

    p = arc(radius=radius, angle=angle, npoints=npoints)
    c = Component()
    path = p.extrude(x)
    ref = c << path
    c.add_ports(ref.ports)
    c.absorb(ref)

    c.info["length"] = float(snap_to_grid(p.length()))
    c.info["dy"] = snap_to_grid(float(abs(p.points[0][0] - p.points[-1][0])))
    c.info["radius"] = float(radius)
    x.validate_radius(radius)

    top = None if int(angle) in {180, -180, -90} else 0
    bottom = 0 if int(angle) in {-90} else None
    x.add_bbox(c, top=top, bottom=bottom)
    if add_pins:
        x.add_pins(c)
    c.add_route_info(
        cross_section=x, length=c.info["length"], n_bend_90=abs(angle / 90.0)
    )
    return c


bend_circular180 = partial(cnm_bend_circular, angle=180)


if __name__ == "__main__":
    from gdsfactory.generic_tech import get_generic_pdk

    PDK = get_generic_pdk()
    PDK.activate()

    c = cnm_bend_circular(
        angle=180,
        cross_section="xs_rc",
        layer=(2, 0),
    )
    c.show(show_ports=True)


__all__ = ["PATH"]

import pathlib

home = pathlib.Path.home()
cwd = pathlib.Path.cwd()

cwd_config = cwd / "config.yml" # TODO: Delete if not needed

home_config = home / ".config"  # TODO: Delete if not needed
config_dir = home / ".config"   # TODO: Delete if not needed
config_dir.mkdir(exist_ok=True)
module_path = pathlib.Path(__file__).parent.absolute()
repo_path = module_path.parent

library_version = "6.1.1"

class Path:
    module = module_path
    repo = repo_path

    tech_dir = module_path / "klayout"
    lyp = module_path / tech_dir / "cnm_klayout_layers.lyp"
    lyt = module_path / tech_dir / "cnm_technology.lyt"

    lyp_2_yaml = module_path / "cnm_layer_views.yaml"

    ip_blocks_dir = module_path / "ip_blocks"
    examples_dir = module_path / "examples"
    components_dir = module_path / "components"
    library_path = ip_blocks_dir / "CNM_library_v6.1.1_BB_MPW_06_07-07-2022_CNMPDK.gds"

    

PATH = Path()

if __name__ == "__main__":
    print(PATH)



from __future__ import annotations

import gdsfactory as gf
from gdsfactory.get_factories import get_cells
from gdsfactory.pdk import Pdk, constants


from cnmpdk import components
from cnmpdk import ip_blocks

from cnmpdk.config import PATH
from cnmpdk.tech import LAYER, cnm_cross_sections
# from cnmpdk.xsections import cross_sections
from cnmpdk.tech import deep, trench, heater, shallow
from functools import partial
from gdsfactory.technology import LayerView, LayerViews

cells = get_cells([components])

PDK = Pdk(
    name="cnmpdk",
    cells=cells,
    cross_sections= cnm_cross_sections,
    layers=LAYER.dict(),
    layer_stack=None,
    layer_views= LayerViews(filepath=PATH.lyp_2_yaml),
    # layer_transitions=LAYER_TRANSITIONS,
    # sparameters_path=PATH.sparameters,
    constants=constants,
)

# TODO: Remove if not needed
gf.routing.all_angle.LOW_LOSS_CROSS_SECTIONS = [
    {"cross_section": "rib", "settings": {"width": 2.5}},
    {"cross_section": "strip", "settings": {"width": 6.0}},
    {"cross_section": "strip", "settings": {"width": 3.0}},
]

PDK.activate()

if __name__ == "__main__":
    # layer_views = LayerViews(filepath=PATH.lyp_yaml)
    # layer_views.to_lyp(PATH.lyp)
    print(PDK.name)


from __future__ import annotations
import gdsfactory as gf
from cnmpdk.config import PATH

@gf.cell
def cnmMMI1x2SH_BB() -> gf.Component:
    """Return cnmMMI1x2SH_BB fixed cell."""
    c = gf.Component()
    c = gf.import_gds(PATH.library_path, "cnmMMI1x2SH_BB")
    c.add_port(name="in0",center=[0,0],width=1.2,orientation=180,layer=1)
    c.add_port(name="out0",center=[91.48900,-4.65],width=1.2,orientation=0,layer=1)
    c.add_port(name="out1",center=[91.48900,4.65],width=1.2,orientation=0,layer=1)
    return c